/*
 * Copyright (c) 2001-2003 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: xmpi_tr_segment.cc,v 1.6 2003/09/03 04:28:54 jsquyres Exp $
 *
 *	Function:	- select and view a trace segment
 */

#include <Xm/SelectioB.h>
#include "lam.h"
#include "xmpi.h"
#include "xmpi_db_parse.h"
#include "xmpi_ctl.h"

/*
 * private functions
 */
static void selcancel_cb(Widget, XtPointer, 
			 XmSelectionBoxCallbackStruct*);
static void selseg_cb(Widget, XtPointer, 
		      XmSelectionBoxCallbackStruct*);

/*
 *	xmpi_tr_select_segment
 *
 *	Function:	- select, parse and view a trace segment
 *	Accepts:	- maximum segment number
 */
void
xmpi_tr_select_segment(int maxseg)
{
	Widget		dialog_w;		/* dialog to select segment */
	XmString	str1, str2;		/* various strings */
	Arg		args[3];
	char		buf[32];		/* formatting buffer */
/*
 * If there is only a single segment then no selection is necessary.
 */
	if (maxseg == 1) {

		if (xmpi_db_parse_segment(0)) {
			xmpi_db_free();
			xmpi_ctl_resetinfo();
		} else {
			xmpi_db_view();
		}

		xmpi_unbusy();
		return;
	}
/*
 * Pop up a dialog to choose a segment number.  The OK callback will
 * parse and view the selected segment.
 */
	sprintf(buf, "Enter segment to display (0-%d)", maxseg - 1);
	str1 = XmStringCreateSimple(buf);
	str2 = XmStringCreateSimple((char*) "0");

	XtSetArg(args[0], XmNselectionLabelString, str1);
	XtSetArg(args[1], XmNautoUnmanage, False);
	XtSetArg(args[2], XmNtextString, str2);
	
	dialog_w = XmCreatePromptDialog(xmpi_shell, 
					(char*) "prompt_pop", args, 3);
	
	XmStringFree(str1);
	XmStringFree(str2);
	XtAddCallback(dialog_w, XmNokCallback, 
		      (XtCallbackProc) selseg_cb, NULL);
	XtAddCallback(dialog_w, XmNcancelCallback, 
		      (XtCallbackProc) selcancel_cb, NULL);
	XtSetSensitive(XmSelectionBoxGetChild(dialog_w, XmDIALOG_HELP_BUTTON),
			False);
	XtManageChild(dialog_w);
	XtPopup(XtParent(dialog_w), XtGrabNone);
}

/*
 *	selseg_cb
 *
 *	Function:	- segment selection callback, parses and views
 *			  the selected segment
 *	Accepts:	- callback parms
 */
static void
selseg_cb(Widget w, XtPointer, 
	  XmSelectionBoxCallbackStruct *cbs)
{
	char		*s;
	int		segment;		/* segment number */

	XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &s);

	segment = (s && *s) ? stoi((unsigned char *)s) : 0;

	XtFree(s);
	XtDestroyWidget(w);

	if (xmpi_db_parse_segment(segment)) {
		xmpi_db_free();
		xmpi_ctl_resetinfo();
	} else {
		xmpi_db_view();
	}

	xmpi_unbusy();
}

/*
 *	selcancel_cb
 *
 *	Function:	- segment selection cancel callback
 *	Accepts:	- callback parms
 */
static void
selcancel_cb(Widget w, XtPointer, 
	     XmSelectionBoxCallbackStruct *)
{
	xmpi_ctl_resetinfo();
	XtDestroyWidget(w);
}
